@@ -61,7 +61,6 @@
 #include "xfdesktop-common.h"
 #include "xfdesktop-thumbnailer.h"
 #include "xfdesktop-settings-ui.h"
-#include "xfdesktop-settings-appearance-frame-ui.h"
 /* for XfceBackdropImageStyle && XfceBackdropColorStyle */
 #include "xfce-backdrop.h"
 
@@ -83,16 +82,6 @@
 #define WINLIST_SHOW_WS_SUBMENUS_PROP        "/windowlist-menu/show-submenus"
 #define WINLIST_SHOW_ADD_REMOVE_WORKSPACES_PROP "/windowlist-menu/show-add-remove-workspaces"
 
-#define DESKTOP_ICONS_ON_PRIMARY_PROP        "/desktop-icons/primary"
-#define DESKTOP_ICONS_STYLE_PROP             "/desktop-icons/style"
-#define DESKTOP_ICONS_ICON_SIZE_PROP         "/desktop-icons/icon-size"
-#define DESKTOP_ICONS_FONT_SIZE_PROP         "/desktop-icons/font-size"
-#define DESKTOP_ICONS_CUSTOM_FONT_SIZE_PROP  "/desktop-icons/use-custom-font-size"
-#define DESKTOP_ICONS_SHOW_TOOLTIP_PROP      "/desktop-icons/show-tooltips"
-#define DESKTOP_ICONS_TOOLTIP_SIZE_PROP      "/desktop-icons/tooltip-size"
-#define DESKTOP_ICONS_SINGLE_CLICK_PROP      "/desktop-icons/single-click"
-#define DESKTOP_ICONS_GRAVITY_PROP           "/desktop-icons/gravity"
-
 typedef struct
 {
     GtkTreeModel *model;
@@ -1743,14 +1732,18 @@ is_our_window(AppearancePanel *panel,
 
         g_message("No good way to match a GdkWindow and XfwWindow for Wayland; guessing");
 
-        if (window_name != NULL && g_strcmp0(window_name, toplevel_name) != 0
-            && gtk_window_is_maximized(toplevel_win) != xfw_window_is_maximized(window))
+        if (window_name != NULL && g_strcmp0(window_name, toplevel_name) == 0
+            && gtk_window_is_maximized(toplevel_win) == xfw_window_is_maximized(window)
+            && ((gdk_window_get_state(gtk_widget_get_window(toplevel)) & GDK_WINDOW_STATE_FULLSCREEN) != 0) == xfw_window_is_fullscreen(window))
         {
             matches = TRUE;
         }
         // TODO: check window geometry?
+    } else
+#endif  /* ENABLE_WAYLAND */
+    {
+        g_assert_not_reached();
     }
-#endif  /* GTK_WINDOWING_WAYLAND */
 
     return matches;
 }
@@ -2061,12 +2054,8 @@ xfdesktop_settings_dialog_setup_tabs(GtkBuilder *main_gxml,
     g_signal_connect(G_OBJECT(screen), "monitors-changed",
                      G_CALLBACK(cb_monitor_changed), panel);
 
-    appearance_gxml = gtk_builder_new();
-    if(!gtk_builder_add_from_string(appearance_gxml,
-                                    xfdesktop_settings_appearance_frame_ui,
-                                    xfdesktop_settings_appearance_frame_ui_length,
-                                    &error))
-    {
+    appearance_gxml = gtk_builder_new_from_resource("/org/xfce/xfdesktop/settings/xfdesktop-settings-appearance-frame-ui.glade");
+    if (G_UNLIKELY(appearance_gxml == NULL)) {
         g_printerr("Failed to parse appearance settings UI description: %s\n",
                    error->message);
         g_clear_error(&error);
@@ -2387,12 +2376,10 @@ main(int argc, char **argv)
         return 1;
     }
 
+    xfdesktop_settings_ui_register_resource();
 
-    gxml = gtk_builder_new();
-    if(!gtk_builder_add_from_string(gxml, xfdesktop_settings_ui,
-                                    xfdesktop_settings_ui_length,
-                                    &error))
-    {
+    gxml = gtk_builder_new_from_resource("/org/xfce/xfdesktop/settings/xfdesktop-settings-ui.glade");
+    if (G_UNLIKELY(gxml == NULL)) {
         g_printerr("Failed to parse UI description: %s\n", error->message);
         g_clear_error(&error);
         return 1;
